/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package opennlp.tools.formats.brat;

public class AttributeAnnotation extends BratAnnotation {

  private final String attachedTo;
  private final String value;

  protected AttributeAnnotation(String id, String type, String attachedTo,
      String value) {
    super(id, type);
    this.attachedTo = attachedTo;
    this.value = value;
  }

  public String getAttachedTo() {
    return attachedTo;
  }

  public String getValue() {
    return value;
  }

  @Override
  public String toString() {
    return super.toString() + " " + attachedTo + (value != null ? " " + value : "");
  }
}
