/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

body {
 margin-top: 1em;
 margin-bottom: 1em;
 margin-left: 16%;
 margin-right: 8%
}

h1, h2, h3, h4, div.toc {
 color: #006699;
}

div.legalnotice {
 max-width: 450px;
}

pre.programlisting, pre.screen, pre.literallayout {
  border: 1px dashed #006699;
  background-color: #EEE;
}

/* 
 * Java syntax highlighting with eclipse default colors
 * and default font-style
 */
pre.programlisting .hl-keyword {
  color: #7F0055;
  font-weight: bold; 
}

/* Seems to be broken, override red inline style of hl-string */
pre.programlisting .hl-string, pre.programlisting b.hl-string i[style]{
  color: #2A00FF !important;
}

pre.programlisting .hl-tag {
  color: #3F7F7F;
}

pre.programlisting .hl-comment {
  color: #3F5F5F;
  font-style: italic;
}

pre.programlisting .hl-multiline-comment {
  color: #3F5FBF;
  font-style: italic;
}

pre.programlisting .hl-value {
  color: #2A00FF;
}

pre.programlisting .hl-attribute {
  color: #7F007F;
}
