/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package opennlp.dl.doccat.scoring;

import java.util.List;

/**
 * Used to determine the scores of document classification. During classification,
 * documents are separated into smaller parts. The classification is run on each part.
 * Implementations of this interface are used to determine the score from the scores
 * of each individual part.
 */
public interface ClassificationScoringStrategy {

  /**
   * Calculate the final scores based on the scores of the individual document parts.
   * @param scores The scores for each individual part of the document.
   * @return The calculated scores.
   */
  double[] score(List<double[]> scores);

}
