/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package opennlp.tools.cmdline;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;

import opennlp.tools.util.InputStreamFactory;

public class SystemInputStreamFactory implements InputStreamFactory {

  private boolean isTainted = false;

  public static Charset encoding() {
    return Charset.defaultCharset();
  }

  @Override
  public InputStream createInputStream() throws IOException {

    if (!isTainted) {
      isTainted = true;
      return System.in;
    }
    else {
      throw new UnsupportedOperationException(
          "The System.in stream can't be re-created to read from the beginning!");
    }
  }
}
