/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package opennlp.tools.chunker;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

/**
 * This is the test class for {@link ChunkerModel}.
 */
public class ChunkerModelTest {

  @Test
  void testInvalidFactorySignature() throws Exception {

    ChunkerModel model = null;
    try {
      model = new ChunkerModel(this.getClass().getResourceAsStream("chunker170custom.bin"));
    } catch (IllegalArgumentException e) {
      Assertions.assertTrue(
          e.getMessage().contains("ChunkerFactory"), "Exception must state ChunkerFactory");
      Assertions.assertTrue(
          e.getMessage().contains("opennlp.tools.chunker.DummyChunkerFactory"),
          "Exception must mention DummyChunkerFactory");
    }
    Assertions.assertNull(model);
  }

  @Test
  void test170DefaultFactory() throws Exception {

    // This is an OpenNLP 1.x model. It should load with OpenNLP 2.x.
    Assertions.assertNotNull(
        new ChunkerModel(this.getClass().getResourceAsStream("chunker170default.bin")));

  }

  @Test
  void test180CustomFactory() throws Exception {

    // This is an OpenNLP 1.x model. It should load with OpenNLP 2.x.
    Assertions.assertNotNull(
        new ChunkerModel(this.getClass().getResourceAsStream("chunker180custom.bin")));

  }
}
